// -*- C++ -*-
// $Header: /u/drspeech/repos/quicknet2/QN_ListStreamHTK.cc,v 1.3 2014/05/06 03:03:32 davidj Exp $

// QN_ListStreamHTK.cc

#include <stdio.h>
#include "QN_ListStreamHTK.h"
#include "QN_utils.h"

////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////
// QN_InFtrStream_ListHTK - class for a list of HTK feat files
////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////
QN_InFtrStream_ListHTK::QN_InFtrStream_ListHTK(int a_debug, 
					       const char* a_dbgname, 
					       FILE* a_file,
					       int a_indexed)
    : QN_InFtrStream_List(a_debug, a_dbgname, a_file, a_indexed,
			  QN_LISTTYPE_PLAIN),
      feat_stream(NULL),
      fp(NULL)
{
    init();			// do base class initialization
}

QN_InFtrStream_ListHTK::~QN_InFtrStream_ListHTK() {
  if(feat_stream != NULL) {	// remove existing feature stream
    delete feat_stream;
  }
  if(fp != NULL)		// close file pointer
    fclose(fp);
}


size_t
QN_InFtrStream_ListHTK::read_ftrs(size_t count, float* ftrs)
{
    size_t cnt = 0;

    if(feat_stream != NULL){
	cnt = feat_stream->read_ftrs(count,ftrs);
	current_frame += cnt;
    } else {
	log.error("Attempting to read features before reading "
		  "the header.");
    }
    return cnt;
}

void
QN_InFtrStream_ListHTK::open_ftrfile()
{
    if(feat_stream != NULL) {	// remove existing feature stream
	delete feat_stream;
    }
    if(fp != NULL)		// close file pointer
	QN_close(fp);

    log.log(QN_LOG_PER_SENT, "Opening HTK file %s", 
	    curr_filename);
    fp = QN_open(curr_filename,"r");
    if(fp == NULL){
	log.error("Error opening HTK feature file: '%s'",curr_filename);
    }

    feat_stream = new QN_InFtrStream_HTK1(debug,dbgname,fp);
    frames_this_seg = feat_stream->num_frames();

    size_t ftrs_this_seg = feat_stream->num_ftrs();
    if (frame_ftrs==QN_SIZET_BAD)
	frame_ftrs = ftrs_this_seg;
    else
    {
	if (frame_ftrs!=ftrs_this_seg)
	{
	    log.error("HTK feature file %s should have %lu features"
		      " but has %lu.", curr_filename,
		      (unsigned long) frame_ftrs,
		      (unsigned long) ftrs_this_seg);
	}
    }
    feat_stream->nextseg();	// prepare for reading the file
}

////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////
// QN_InFtrStream_ListSCP - class for a list of HTK feat files with
//                          utterance IDS and start and end frames.
////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////
QN_InFtrStream_ListSCP::QN_InFtrStream_ListSCP(int a_debug, 
					       const char* a_dbgname, 
					       FILE* a_file,
					       int a_indexed)
    : QN_InFtrStream_List(a_debug, a_dbgname, a_file, a_indexed,
			  QN_LISTTYPE_SCP),
      feat_stream(NULL),
      fp(NULL)
{
    init();			// do base class initialization
}

QN_InFtrStream_ListSCP::~QN_InFtrStream_ListSCP() {
    if(feat_stream != NULL) {	// remove existing feature stream
	delete feat_stream;
    }
    if(fp != NULL)		// close file pointer
	fclose(fp);
}

size_t
QN_InFtrStream_ListSCP::read_ftrs(size_t count, float* ftrs)
{
    size_t cnt = 0;

    if(feat_stream != NULL){
	cnt = feat_stream->read_ftrs(count,ftrs);
	current_frame += cnt;
    } else {
	log.error("Attempting to read features before reading "
		  "the header.");
    }
    return cnt;
}

void
QN_InFtrStream_ListSCP::open_ftrfile()
{
    if(feat_stream != NULL) {	// remove existing feature stream
	delete feat_stream;
    }
    if(fp != NULL)		// close file pointer
	QN_close(fp);

    log.log(QN_LOG_PER_SENT, "Opening HTK file %s", 
	    curr_filename);
    fp = QN_open(curr_filename,"r");
    if(fp == NULL){
	log.error("Error opening HTK feature file: '%s'",curr_filename);
    }

    feat_stream = new QN_InFtrStream_HTK1(debug,dbgname,fp, curr_index1,
					  curr_index2);
    frames_this_seg = feat_stream->num_frames();
    size_t ftrs_this_seg = feat_stream->num_ftrs();
    if (frame_ftrs==QN_SIZET_BAD)
	frame_ftrs = ftrs_this_seg;
    else
    {
	if (frame_ftrs!=ftrs_this_seg)
	{
	    log.error("HTK feature file %s should have %lu features"
		      " but has %lu.", curr_filename,
		      (unsigned long) frame_ftrs,
		      (unsigned long) ftrs_this_seg);
	}
    }
    feat_stream->nextseg();	// prepare for reading the file
}

