// $Header: /u/drspeech/repos/quicknet2/QN_MLPWeightFile_Tnet.h,v 1.3 2014/02/03 20:49:46 davidj Exp $

#ifndef QN_MLPWeightFile_Tnet_h_INCLUDED
#define QN_MLPWeightFile_Tnet_h_INCLUDED

/* Must include the config.h file first */
#include <QN_config.h>
#include <stdio.h>
#include "QN_types.h"
#include "QN_MLPWeightFile.h"
#include "QN_mat.h"

// A weight file format that's a subset of the weight files
// used by the TNet MLP package.   Works for up deep MLPs.



class QN_MLPWeightFile_Tnet : public QN_MLPWeightFile
{
public:
    enum {
	QN_MAX_LAYERS = QN_MLP_MAX_LAYERS,
	QN_MAX_WEIGHTMATS = QN_MAX_LAYERS-1,
	QN_MAX_SECTIONS = QN_MAX_LAYERS-1 + QN_MAX_WEIGHTMATS
    };

    // If layer sizes are non-zero for input, the file is checked to
    // see if they are consistent.  
    // The "skip_sections" arg specifies how many leading sections of the
    // weight file to skip that may specify e.g. windowing or normalization.
    // The out_layer_type is needed for writing TNet-style files.
    QN_MLPWeightFile_Tnet(int a_dbg, const char* a_dbgname, FILE* a_stream,
			  QN_FileMode a_mode, size_t a_layers,
			  const size_t* a_layer_units = NULL,
			  QN_OutputLayerType a_out_layer_type
			                    = QN_OUTPUT_SOFTMAX,
			  size_t a_skip_sections = 0);
    virtual ~QN_MLPWeightFile_Tnet();

    // How many layers are there?
    size_t num_layers();

    // How many sections?
    size_t num_sections();

    // How many units are there in the given layer?
    size_t size_layer(QN_LayerSelector layer);

    // Return whether weight matrices are input or output major
    enum QN_WeightMaj get_weightmaj();

    // Return the order of the weights in the file
    enum QN_SectionSelector get_weighttype(int section);

    // Is this fine QN_READ or QN_WRITE?
    enum QN_FileMode get_filemode();

    // Read in a buffer full of weights
    // Note that it is illegal to read across weight matrix boundaries.
    size_t read(float* dest, size_t count);

    // Write out a buffer full of weights
    size_t write(const float* buf, size_t count);

private:
    // Handle the stuff at the beginning a given weights section
    void do_weights_header(size_t layer, size_t next_state);
    // Handle the stuff at the beginning a given biases section
    void do_bias_header(size_t layer, size_t next_state);
    // A function to write the header info at the start
    // of the next section and set everything up for that section
    void write_section_header(void);
    // Scan file, getting layer sizes.
    void scan_file(void);
    // Go to the start of a given layer.
    void seek_layer_header(void);
    // Skip over header of biases.
    void skip_bias_header(void);
    // Given an output layer type, return the string for the TNet weight
    // file section name.
    const char* layer_string(QN_OutputLayerType type);

    // Read a character, error if EOF.
    int mygetc_eofbad();
    // Read a charcater, no error if EOF.
    int mygetc_eofok();
    // Put a character back on the input sream.
    void myungetc(int c);
    // Skip over white space, error if EOF.
    void myskipspace_eofbad(void);
    // Return an integer size from the intput file, error if EOF.
    unsigned long mygetsize_eofbad();
    // Return a double from the intput file, error of EOF.
    double mygetdouble_eofbad();
    // Return the current file position.
    void mygetpos(fpos_t* pos);
    
private:
    struct QN_Tnet_layer
    {
	size_t n_output;
	size_t n_input;
	QN_LayerType layer_type;
	fpos_t pos;
	unsigned long lineno;
    };

    // Names of the section matrices.
    QN_ClassLogger clog;	// Handles logging
    FILE* const stream;		// Stream used to access weights files
    const enum QN_FileMode mode;


    size_t n_layers;		// Number of layers in the net.
    size_t layer_units[QN_MAX_LAYERS]; // The size of each layer.
    QN_OutputLayerType out_layer_type; // Output layer type.

    size_t skip_sections;	       // How many sections to skip when
				       // reading.

// Writing vars.
    size_t output_state;	       // What section we are in.
    size_t output_outputs;	// Number of outputs in the current section.
    size_t output_inputs;	// Number of inputs in the current section.
    size_t output_row_count;	// Number of rows remaining.
    size_t output_col_count;	// Number of columns remaining

// Reading vars.
    QN_Tnet_layer tnet_layers[QN_MAX_LAYERS+1]; // TNet description of each
						// layer - first one empty.
    size_t cur_input_layer;			// Current input layer,
                                                // 1 is first layer.
    size_t input_count;				// Count of floats left to
						// read.
    size_t input_biases;			// True if reading biases.
    size_t lineno;				// Current line number for
						// reading.
};

#endif

