. $Header: /u/drspeech/repos/quicknet2/qngentrn.man,v 1.6 2014/05/06 02:06:47 davidj Exp $
.TH qngentrn 1 "$Date: 2014/05/06 02:06:47 $" ICSI "ICSI SPEECH SOFTWARE"
.SH NAME
qngentrn \- layer-by-layer MLP training program
.SH SYNOPSIS
.B qngentrn
[
.I options
]
.SH DESCRIPTION
.I qngentrn
is a program for layer-by-layer ("generational") training of deep Multi Layer
Perceptrons (MLPs).  E.g. for training a 153x1000x2000x1000x56
net, it will train a 153x1000x56 net, then train a 153x1000x2000x56
net then, finally, a 153x1000x2000x1000x56 net. 
It is particularly useful for training deep MLPs for
speech recognition tasks.
.P
.I qngentrn
is a python script and uses
.I qnmultitrn(1)
to train each generation and
.I qnmungewts(1)
to modify the intermediate weight files.
.PD 1
.SH OPTIONS
Command line options can
optionally have a double dash prefix, e.g.:
.TP
.B ftr1_file=filename1
.P
or
.TP
.B --ftr1_file=filename1
.P
.I qngentrn
can take the following options:
.TP
.BI ftr1_file= filename1
Specify the file containing feature information for training the
net.  The features in the file are normalized, selected (to eliminate
unwanted features) and windowed before being combined to
form the input to the MLP.  
.TP
.BI ftr1_format= format
Specify the format of the feature file.  The format can be
\fBpfile\fR (the ICSI feature file format - see \fBpfile\fR(5)),
\fBonlftr\fR (the ICSI online feature file format),
\fBsrifile\fR (the SRI feature file format),
\fBsrilist\fR (a file containing a list of SRI feature files),
\fBhtklist\fR (a file containing a list of HTK feature files),
\fBhtkscp\fR (a file containing HTK segids, filenames, and frame start/end),
\fBlna\fR (the Cambridge compressed activation file format),
or \fBpre\fR (the Cambridge compressed feature file format).  The
default is \fBpfile\fR.
.TP
.BI ftr1_width= integer
Specify the number of features in the feature file.  The
default \- \fB0\fR
\- means use the number of features specified in the feature file header.
A non-zero value must be given for feature files that are in a format
that does not include a header, e.g. \fBpre\fR files.
.TP
.BI hardtarget_file= filename
Specify the file containing label information for specifying the
target when training the net.  This file can be the same as 
ftr1_file.  The target file must contain only one label per
frame.
.TP
.BI hardtarget_format= format
Specify the format of the file containing target labels.  The format can be
\fBpfile\fR (the ICSI feature file format - see \fBpfile\fR(5)),
\fBpre\fR (the Cambridge compressed feature file format) or 
\fBilab\fR (the ICSI compressed label file format).  The
default is \fBpfile\fR.  If the \fBpre\fR format is specified
\fBhardtarget_file\fR and \fBftr1_file\fR must be the same.
.TP
.BI ftr1_norm_file= filename1
Specify a file containing normalization parameters for the values in
the feature file.  The normalization values are used
for both training and cross validation.  A null string results in no
normalization.  See \fBftr1_norm_mode\fR below.
.TP
.BI ftr1_ftr_start= integer
Specify the column number of the first feature to use from the
feature file (possibly augmented by delta_order).  
\fB0\fR means start with the first
feature column.
.TP
.BI ftr1_ftr_count= integer
Specify the number of feature to use from the feature
file (including added feature columns).  
The default \- \fB0\fR \- means use all remaining features.
.TP
.BI hardtarget_lastlab_reject= bool
Usually if the net has \fIn\fR outputs, then the valid range of hardtarget
label values is \fI0\fR through \fIn - 1\fR.  However, if
\fBhardtarget_lastlab_reject\fR is set to \fBtrue\fR then an extra
label value is allowed.  A label of value \fBn\fR will mean "do not
train on this label".  This prevents the features from being presented
to the net and also means the frame will be ignored when
calculating cross-validation accuracy.
.TP
.BI window_extent= integer
Specify the number of frames  from the beginning of the first input
window to the end of the last input window.  Typically this is the
same as \fBftr1_window_len\fR.
.TP
.BI ftr1_window_offset= integer
Specify the offset, in frames, of the window on the feature file from
the base of the overall window.  Typically,
\fBftr1_window_offset\fR is \fB0\fR.
.TP
.BI hardtarget_window_offset= integer
Specify the offset, in frames, of the window on the label file from
the base of the overall window.  Note that the length of the label
file window is always \fB1\fR for normal hard training.
.TP
.BI ftr1_window_len= integer
Specify the length, in frames, of the window on the feature file.
Often, the value of \fBftr1_window_len\fR will be the same as
the value of \fBwindow_extent\fR.
.TP
.BI ftr1_delta_order= integer
Specify the order of delta-calculations to be applied to the
feature file.  Currently, this is applied 
.I before
feature selection 
via \fBftr1_start\fR and \fBftr1_count\fR, effectively 
doubling or tripling the width of the 
input feature stream.  \fBftr1_start\fR and \fBftr1_count\fR 
can then be used to select columns from the net aggregate feature set.
Acceptable values are 0 (no action), 1 (just deltas) 
and 2 (deltas and double-deltas). 
Note that the filters are not scaled by the sum of squares, so
mean-variance normalization is recommended.
.TP
.BI ftr1_delta_win= integer
Number of points in the FIR discrete-differentiators used to 
calculate deltas and double deltas.  Default value is 9.
.TP
.BI ftr1_norm_mode= mode
Specifies normalizataion applied to the feature file.
Options include "file" for constant per-feature normalization stored
in a file.  Setting e.g.
\fBftr1_norm_mode\fR to "file" and setting \fBftr1_norm_file\fR to the
null string results in no normalization.  See 
qncopy(1) for more description. 
.TP
.BI train_cache_frames= integer
The number of frames of training data loaded into the cache used for
presentation randomization.  A variable number of sentences is read
sequentially from the training feature file until this cache is as
close to being full as possible.  This cache is then used to provide
random presentations for training until all frames have been used
once, at which point the next sequence of sentences is loaded.  The
number of frames cached has a significant affect on memory usage, with
one frame requiring approximately (ftr_count+lab_count+1)*4 bytes.
.TP
.BI train_cache_seed= integer
Set the seed for random training pattern selection.
.TP
.BI train_sent_range= range-spec-string
The sentences to use for training, specified by a Range token in 
one of the formats defined by QN_Range(3).  
.TP
.BI cv_sent_range= range-spec-string
The sentences to use for cross validation, specified by a Range token in 
one of the formats defined by QN_Range(3).
.TP
.BI out_weight_file= filename
Specify a file in which to save the weights from the trained net.
.TP
.BI log_weight_file= filename
Specify a file in which a copy of the weights are saved at the end of
each epoch.  This file is used when the cross validation error rate for an
epoch is greater than the previous one, in which case the latest weight
updates are abandoned by reloading the weights from this file. See
below for details on using % characters in weight file names.
.TP
.BI log2_weight_file= filename
Specify a file in which a copy of the initial weights for each
training generation is stored.
.TP
.BI log3_weight_file= filename
Specify a file in which a copy of the output weights for each
training generation is stored (with the exception of the final training, when
they are stored in \fIout_weight_file\fR).
.P
Six special substrings can be used in the log_weightfile filenames. The string
.B %p
is replaced by the process number of the qngentrn process.  The string
.B %e
is replaced by the number of the current epoch (allowing weight
logs from all epochs to be retained).
The string
.B %t
is replaced by the date and time as would be produced by the Unix
command \fBdate +'%Y%m%d-%H:%M\fR.
The string
.B %h
is replaced by the hostname of the running machine with all 
characters from the first period removed.
The string
.B %g
is replaced by the the generation number, starting with 1 for the
first generation.
.B %%
is replaced by a single
.BR % "."
.TP
.BI out_weight_format= format
Specify the format of the output weight file.  Currently the only
format supported is 
\fBmatlab\fR (encoded as old-style Matlab format float matrices).
.TP
.BI log_weight_format= format
Specify the format of the various log weight files.  Currently the only
format supported is 
\fBmatlab\fR (encoded as old-style Matlab format float matrices).
.P
.PD 0
.BI init_random_bias_min= float[,float][,float...]
.P
.BI init_random_bias_max= float[,float][,float...]
.P
.BI init_random_weight_min= float[,float][,float...]
.TP
.PD 1
.BI init_random_weight_max= float,[float][,float...]
Limits for the random initialization of weights and biases.  If one value
supplied, applies to all weights/biases.  If two values supplied,
second applies to the output layer and the first to the rest of the
layers.  If more than two values
are supplied, there must be one fewer values than the number of layers
(due to the input layer not having biases or weights).  In this case, values
apply to layers in order of input through output.
.TP
.BI init_random_seed= integer
Set the seed for random number generation used for
initialization of weights.
.TP
.BI learnrate_schedule= schedule
The name of the strategy used for updating the learning rate in
the final training of the full net.

\fInewbob\fR means use the same approach as taken by BoB: use a
constant learning rate until the error reduction drops below a given
threshold, then decrease exponentially.  The initial learning rate is
set the first value specified in the \fBlearnrate_vals\fR option.  The
ratio of successive learning rates when decaying is set by
\fRlearnrate_scale\fB. 

\fIlist\fR means use the learning rate list specified in the
\fBlearnrate_vals\fR option.  After all learning rates have been used, the
training terminates.

The default value is \fInewbob\fB.
.TP
.BI learnrate_vals= float,float,...
Specify the learning rate for one or more epochs in the final
training of the full net.  See the
\fBlearnrate_schedule\fR option for more details.
.TP
.BI learnrate_scale= float
Specify the scaling factor of new learning rates to old learning
rates during learning rate decay for the final training of the full net.  See
\fBlearnrate_schedule\fR option for more details.
.TP
.BI learnrate_epochs= integer
Maximum number of epochs of training for the final training of the
full net.  We stop when we have trained
this many epochs regardless of the learning rate schedule.  if
\fBlearnrate_epochs\fR is 0, we do not train at all.
.TP
.BI gen_learnrate_schedule= schedule
The name of the strategy used for updating the learning rate in
the layer-by-layer trainings of the partial nets.
See \flearnrate_schedule\fR option for the available schedules.
The default value is \fInewbob\fB.
.TP
.BI gen_learnrate_vals= float,float,...
Specify the learning rate for one or more epochs in the layer-by-layer
trainings of the partial nets.  See the
\fBlearnrate_schedule\fR option for more details.
.TP
.BI gen_learnrate_scale= float
Specify the scaling factor of new learning rates to old learning
rates during learning rate decay for the layer-by-layer trainings of
the partial nets.  See
\fBlearnrate_schedule\fR option for more details.
.TP
.BI gen_learnrate_epochs= integer
Maximum number of epochs of training for the layer-by-layer trainings
of the partial nets.  We stop when we have trained
this many epochs regardless of the learning rate schedule.  if
\fBlearnrate_epochs\fR is 0, we do not train at all.
.TP
.BI mlp_input_size= integer
Specify the size of the input layer for all of the nets.
.TP
.BI mlp_hidden_size= integer,integer[,...]
Specify the size of the hidden layers for the final net, with the layer
closest to the input layer listed specified
first.  The number of
integers implies the number of hidden layers, which can be between 1 and 7.
.TP
.BI mlp_output_size= integer
Specify the size of the output layer for all of the nets.
.TP
.BI mlp_hidden_type= unittype
Specify the type of non-linearity to use for the MLP hidden layers.
Allowable values are \fBsigmoid\fR (the logstic function, the
default), \fBtanh\fR
and \fBrelu\fR (a rectified linear function with unit slope).  
.TP
.BI mlp_output_type= unittype
Specify the type of non-linearity to use for the MLP output layer.
Allowable values are \fBsigmoid\fR, \fBsigmoidx\fR (sigmoid with cross
entropy error criteria during training), \fBtanh\fR and \fBsoftmax\fR
(the default). 
.TP
.BI use_pp= bool
Use high-performance internal matrix routines for the MLP if
.BR true .
This is enabled by default and turning it off is
only really useful for debugging or
performance tuning.  Note that the transcendental routines are only
approximations and so there are slight numerical differences in the
result depending on how this option is set.
.TP 
.BI use_blas= bool
Use blas matrix routines for the MLP if
.BR true .
Setting this to true will generate an error if there is no BLAS
library linked with the executable.  Note that having
use_blas and use_pp both true is reasonable as some internal
optimized routines are not available in the BLAS library.
.TP 
.BI use_fe= bool
Use fast exponent approximation for e.g. sigmoid and softmax functions
if
.BR true .
There are some situations where the fast exponent approximation has
been shown to reduce training accuracy.  However, not using fast
exponent approximation off can result in a lower
connection-updates-per-second number.  The
default is
.BR false .
.TP 
.BI use_cuda= bool
Use any available CUDA hardware if
.BR true .
If no CUDA hardware is available, the program will halt.
.TP 
.BI mlp_bunch_size= integer
Set the number of presentations between forward and backward phases
during MLP training.  A value of \fB0\fR (the default) results in
traditional online training.  \fB1\fR means to use the bunch-mode 
routines, but with a bunch size of 1, which should be numerically very
similar to online training but may have different speed
characteristics.  The default value is 16, which will work well for
most datasets.  Larger bunch sizes will typically result in higher
connection update rates but beyond a dataset-specific limit the convergence
characteristics of the net may be affected.
.TP
.BI mlp_threads= integer
Set the number of CPU threads to use.  Note that this only works for a
bunch size >1. For good performance, the number of threads should be
a small fraction of the bunch size and less than or equal to the
number of available physical cores.  Not compatible with \fIuse_cuda=true\FR.
.TP
.BI log_file= filename
The file in which to log status messages.  Specifying a
filename of
.B \-
sends the results to standard output.
.TP
.BI verbose= bool
Output more status messages if
.BR true .
This can be abbreviated as \fI-v\fR.
.TP
.BI debug= integer
Set the level of internal debugging output.  \fB0\fR means none,
\fB6\fR means lots!  This can be abbreviated as \fI-d\fR.
.TP
.BI --version
Output a version message and exit. This can be abbreviated as \fI-V\fR.
.TP
.BI --help
Output a help message and exit. This can be abbreviated as \fI-h\fR.

.SH ENVIRONMENT
.TP 20
.B TZ
Time Zone.  On some systems, this is used for displaying times during
the training run \- if times seem to be wrong by several hours, it is
because this environment variable is not set.

.SH AUTHOR
David Johnson  <davidj@ICSI.Berkeley.EDU>
.SH SEE ALSO
.BR qnmulttrn (1) ,
.BR qnmungewts (1) ,
.BR qnmultifwd (1) ,
.BR qnnorm (1) ,
.BR QN_Range (3) , 
.BR norms (5) ,
.BR pfile (5) ,
.BR ilab (5) .
.SH BUGS




